"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setCertificateVerifyProc = exports.attachTo = void 0;
const certificateUtils = require("@wireapp/certificate-check");
const electron_1 = require("electron");
const fs = require("fs-extra");
const path = require("path");
const locale_1 = require("../locale/locale");
const getLogger_1 = require("../logging/getLogger");
const EnvironmentUtil = require("../runtime/EnvironmentUtil");
const logger = getLogger_1.getLogger(path.basename(__filename));
var CertificateVerificationResult;
(function (CertificateVerificationResult) {
    CertificateVerificationResult[CertificateVerificationResult["SUCCESS"] = 0] = "SUCCESS";
    CertificateVerificationResult[CertificateVerificationResult["FAILURE"] = -2] = "FAILURE";
    CertificateVerificationResult[CertificateVerificationResult["USE_CHROMIUM_VALIDATION"] = -3] = "USE_CHROMIUM_VALIDATION";
})(CertificateVerificationResult || (CertificateVerificationResult = {}));
let CertificateVerifyProcManager = (() => {
    class CertificateVerifyProcManager {
        static displayCertificateDetails(hostname, certificate, options) {
            return __awaiter(this, void 0, void 0, function* () {
                const goBack = () => __awaiter(this, void 0, void 0, function* () {
                    yield this.displayCertificateError(hostname, certificate, Object.assign(Object.assign({}, options), { bypassDialogLock: true }));
                });
                const textDetails = `${options.isChromiumError ? this.LOCALE.SHOW_DETAILS_TEXT_CHROMIUM : this.LOCALE.SHOW_DETAILS_TEXT_PINNING} ${hostname}`;
                const isTrustDialogSupported = EnvironmentUtil.platform.IS_MAC_OS;
                if (isTrustDialogSupported) {
                    yield electron_1.dialog.showCertificateTrustDialog(this.mainWindow, {
                        certificate,
                        message: textDetails,
                    });
                    yield goBack();
                }
                else {
                    const { response } = yield electron_1.dialog.showMessageBox(this.mainWindow, {
                        buttons: [this.LOCALE.SHOW_DETAILS_GO_BACK, this.LOCALE.SHOW_DETAILS_SAVE_CERTIFICATE],
                        cancelId: this.RESPONSE.GO_BACK,
                        detail: textDetails,
                        message: this.LOCALE.SHOW_DETAILS_TITLE,
                        type: 'info',
                    });
                    switch (response) {
                        case this.RESPONSE.SAVE_CERTIFICATE: {
                            const { filePath: chosenPath } = yield electron_1.dialog.showSaveDialog(this.mainWindow, {
                                defaultPath: `${hostname}.pem`,
                            });
                            if (chosenPath !== undefined) {
                                yield fs.writeFile(chosenPath, Buffer.from(certificate.data));
                            }
                            yield this.displayCertificateDetails(hostname, certificate, options);
                            break;
                        }
                        case this.RESPONSE.GO_BACK: {
                            yield goBack();
                            break;
                        }
                    }
                }
            });
        }
        static isCertificatePinningEnabled() {
            return !this.bypassCertificatePinning;
        }
        static displayCertificateChromiumError(hostname, certificate) {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.displayCertificateError(hostname, certificate, { isChromiumError: true });
            });
        }
        static displayCertificateError(hostname, certificate, options) {
            return __awaiter(this, void 0, void 0, function* () {
                const { bypassDialogLock, isChromiumError, isCheckboxChecked } = Object.assign({ bypassDialogLock: false, isCheckboxChecked: false, isChromiumError: false }, options);
                if (this.isDialogLocked && !bypassDialogLock) {
                    return;
                }
                this.isDialogLocked = true;
                const { checkboxChecked, response } = yield electron_1.dialog.showMessageBox(this.mainWindow, {
                    buttons: [this.LOCALE.RETRY, this.LOCALE.SHOW_DETAILS],
                    cancelId: this.RESPONSE.RETRY,
                    checkboxChecked: isChromiumError ? undefined : isCheckboxChecked,
                    checkboxLabel: isChromiumError ? undefined : this.LOCALE.WARNING_BYPASS,
                    defaultId: this.RESPONSE.RETRY,
                    detail: isChromiumError ? this.LOCALE.WARNING_TEXT_CHROMIUM : this.LOCALE.WARNING_TEXT_PINNING,
                    message: this.LOCALE.WARNING_TITLE,
                    type: 'warning',
                });
                switch (response) {
                    case this.RESPONSE.RETRY: {
                        if (!isChromiumError) {
                            this.bypassCertificatePinning = checkboxChecked;
                            if (this.bypassCertificatePinning) {
                                logger.log('User disabled certificate pinning');
                            }
                        }
                        setTimeout(() => (this.isDialogLocked = false), this.dialogUnlockTimeout);
                        break;
                    }
                    case this.RESPONSE.SHOW_DETAILS: {
                        yield this.displayCertificateDetails(hostname, certificate, {
                            bypassDialogLock,
                            isCheckboxChecked: checkboxChecked,
                            isChromiumError,
                        });
                        break;
                    }
                }
            });
        }
    }
    CertificateVerifyProcManager.bypassCertificatePinning = false;
    CertificateVerifyProcManager.isDialogLocked = false;
    CertificateVerifyProcManager.dialogUnlockTimeout = 6000;
    CertificateVerifyProcManager.CHROMIUM_ERRORS = {
        CERT_AUTHORITY_INVALID: -202,
        CERT_COMMON_NAME_INVALID: -200,
    };
    CertificateVerifyProcManager.RESPONSE = {
        GO_BACK: 0,
        RETRY: 0,
        SAVE_CERTIFICATE: 1,
        SHOW_DETAILS: 1,
    };
    CertificateVerifyProcManager.LOCALE = {
        RETRY: locale_1.getText('certificateVerifyProcManagerRetry'),
        SHOW_DETAILS: locale_1.getText('certificateVerifyProcManagerShowDetails'),
        SHOW_DETAILS_GO_BACK: locale_1.getText('certificateVerifyProcManagerShowDetailsGoBack'),
        SHOW_DETAILS_SAVE_CERTIFICATE: locale_1.getText('certificateVerifyProcManagerShowDetailsSaveCertificate'),
        SHOW_DETAILS_TEXT_CHROMIUM: locale_1.getText('certificateVerifyProcManagerShowDetailsTextChromium'),
        SHOW_DETAILS_TEXT_PINNING: locale_1.getText('certificateVerifyProcManagerShowDetailsTextPinning'),
        SHOW_DETAILS_TITLE: locale_1.getText('certificateVerifyProcManagerShowDetailsTitle'),
        WARNING_BYPASS: locale_1.getText('certificateVerifyProcManagerWarningBypass'),
        WARNING_TEXT_CHROMIUM: locale_1.getText('certificateVerifyProcManagerWarningTextChromium'),
        WARNING_TEXT_PINNING: locale_1.getText('certificateVerifyProcManagerWarningTextPinning'),
        WARNING_TITLE: locale_1.getText('certificateVerifyProcManagerWarningTitle'),
    };
    return CertificateVerifyProcManager;
})();
exports.attachTo = (main) => {
    CertificateVerifyProcManager.mainWindow = main;
};
exports.setCertificateVerifyProc = (request, cb) => __awaiter(void 0, void 0, void 0, function* () {
    const { hostname, certificate, verificationResult, errorCode } = request;
    if (verificationResult !== 'net::OK') {
        logger.error(`Internal Chrome TLS verification failed. Hostname: ${hostname}. Verification result: ${verificationResult}. Error code: ${errorCode}`);
        const isCommonCertificateError = errorCode === CertificateVerifyProcManager.CHROMIUM_ERRORS.CERT_COMMON_NAME_INVALID ||
            errorCode === CertificateVerifyProcManager.CHROMIUM_ERRORS.CERT_AUTHORITY_INVALID;
        if (isCommonCertificateError) {
            yield CertificateVerifyProcManager.displayCertificateChromiumError(hostname, certificate);
        }
        return cb(CertificateVerificationResult.FAILURE);
    }
    if (certificateUtils.hostnameShouldBePinned(hostname) && CertificateVerifyProcManager.isCertificatePinningEnabled()) {
        const pinningResults = certificateUtils.verifyPinning(hostname, certificate);
        const falsyValue = Object.values(pinningResults).some(val => val === false);
        if (falsyValue || pinningResults.errorMessage) {
            logger.error(`Certificate verification failed for "${hostname}".`);
            logger.error(`Error: "${pinningResults.errorMessage}". Displaying certificate pinning error dialog.`);
            yield CertificateVerifyProcManager.displayCertificateError(hostname, certificate);
            return cb(CertificateVerificationResult.FAILURE);
        }
    }
    return cb(CertificateVerificationResult.USE_CHROMIUM_VALIDATION);
});
//# sourceMappingURL=CertificateVerifyProcManager.js.map