"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const Menu = electron_1.remote.Menu;
const eventType_1 = require("../../lib/eventType");
const locale = require("../../locale/locale");
const config_1 = require("../../settings/config");
let textMenu;
let copyContext = '';
const defaultMenu = Menu.buildFromTemplate([
    {
        click: () => electron_1.clipboard.writeText(copyContext),
        label: locale.getText('menuCopy'),
    },
]);
const textMenuTemplate = [
    {
        label: locale.getText('menuCut'),
        role: 'cut',
    },
    {
        label: locale.getText('menuCopy'),
        role: 'copy',
    },
    {
        label: locale.getText('menuPaste'),
        role: 'paste',
    },
    {
        type: 'separator',
    },
    {
        label: locale.getText('menuSelectAll'),
        role: 'selectAll',
    },
];
const createTextMenu = () => {
    const template = textMenuTemplate.slice();
    textMenu = Menu.buildFromTemplate(template);
};
const imageMenu = Menu.buildFromTemplate([
    {
        click: () => savePicture(imageMenu.image || '', imageMenu.timestamp),
        label: locale.getText('menuSavePictureAs'),
    },
]);
window.addEventListener('contextmenu', event => {
    const element = event.target;
    copyContext = '';
    if (element.nodeName === 'TEXTAREA' || element.nodeName === 'INPUT') {
        event.preventDefault();
        createTextMenu();
        textMenu.popup({ window: electron_1.remote.getCurrentWindow() });
    }
    else if (element.classList.contains('image-element') || element.classList.contains('detail-view-image')) {
        event.preventDefault();
        const elementSource = element.src;
        const parentElement = element.closest('.message-body');
        const timeElement = parentElement.getElementsByTagName('time')[0];
        if (timeElement) {
            const imageTimestamp = timeElement.dataset.timestamp;
            imageMenu.timestamp = imageTimestamp;
        }
        imageMenu.image = elementSource;
        imageMenu.popup({ window: electron_1.remote.getCurrentWindow() });
    }
    else if (element.nodeName === 'A') {
        event.preventDefault();
        const elementHref = element.href;
        copyContext = elementHref.replace(/^mailto:/, '');
        defaultMenu.popup({ window: electron_1.remote.getCurrentWindow() });
    }
    else if (element.classList.contains('text')) {
        event.preventDefault();
        copyContext = (window.getSelection() || '').toString() || element.innerText.trim();
        defaultMenu.popup({ window: electron_1.remote.getCurrentWindow() });
    }
    else {
        let parentNode = element.parentNode;
        while (parentNode && parentNode !== document && !parentNode.classList.contains('text')) {
            parentNode = parentNode.parentNode;
        }
        if (parentNode !== document) {
            event.preventDefault();
            copyContext = (window.getSelection() || '').toString() || parentNode.innerText.trim();
            defaultMenu.popup({ window: electron_1.remote.getCurrentWindow() });
        }
    }
}, false);
const savePicture = (url, timestamp) => __awaiter(void 0, void 0, void 0, function* () {
    const response = yield fetch(url, {
        headers: {
            'User-Agent': config_1.config.userAgent,
        },
    });
    const arrayBuffer = yield response.arrayBuffer();
    return electron_1.ipcRenderer.send(eventType_1.EVENT_TYPE.ACTION.SAVE_PICTURE, new Uint8Array(arrayBuffer), timestamp);
});
//# sourceMappingURL=context.js.map