"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SingleSignOn = void 0;
const minimist = require('minimist');
const commons_1 = require("@wireapp/commons");
const crypto = require("crypto");
const electron_1 = require("electron");
const path = require("path");
const url_1 = require("url");
const getLogger_1 = require("../logging/getLogger");
const config_1 = require("../settings/config");
const lifecycle_1 = require("../runtime/lifecycle");
const argv = minimist(process.argv.slice(1));
const LOG_DIR = path.join(electron_1.app.getPath('userData'), 'logs');
let SingleSignOn = (() => {
    class SingleSignOn {
        constructor(mainBrowserWindow, senderEvent, windowOriginURL, windowOptions) {
            this.init = () => __awaiter(this, void 0, void 0, function* () {
                this.session = electron_1.session.fromPartition(SingleSignOn.SSO_SESSION_NAME, { cache: false });
                this.session.setPermissionRequestHandler((_webContents, _permission, callback) => callback(false));
                this.session.webRequest.onBeforeSendHeaders(({ requestHeaders }, callback) => {
                    requestHeaders['User-Agent'] = config_1.config.userAgent;
                    callback({ cancel: false, requestHeaders });
                });
                this.ssoWindow = this.createBrowserWindow();
                yield SingleSignOn.protocol.register(this.session, type => this.finalizeLogin(type));
                yield this.ssoWindow.loadURL(this.windowOriginUrl.toString());
                if (argv[config_1.config.ARGUMENT.DEVTOOLS]) {
                    this.ssoWindow.webContents.openDevTools({ mode: 'detach' });
                }
            });
            this.createBrowserWindow = () => {
                delete this.windowOptions.webPreferences.preloadURL;
                delete this.windowOptions.webPreferences.preload;
                delete this.windowOptions.webPreferences.openerId;
                const ssoWindow = new electron_1.BrowserWindow(Object.assign(Object.assign({}, this.windowOptions), { alwaysOnTop: true, backgroundColor: '#FFFFFF', fullscreen: false, fullscreenable: false, height: this.windowOptions.height || 600, maximizable: false, minimizable: false, modal: false, movable: false, parent: this.mainBrowserWindow, resizable: false, title: SingleSignOn.getWindowTitle(this.windowOriginUrl.origin), titleBarStyle: 'default', useContentSize: true, webPreferences: Object.assign(Object.assign({}, this.windowOptions.webPreferences), { allowRunningInsecureContent: false, backgroundThrottling: false, contextIsolation: true, devTools: true, disableBlinkFeatures: '', enableBlinkFeatures: '', experimentalFeatures: false, images: true, javascript: true, nativeWindowOpen: false, nodeIntegration: false, nodeIntegrationInWorker: false, offscreen: false, partition: '', plugins: false, preload: SingleSignOn.PRELOAD_SSO_JS, sandbox: true, scrollBounce: true, session: this.session, textAreasAreResizable: false, webSecurity: true, webgl: false, webviewTag: false }), width: this.windowOptions.width || 480 }));
                this.senderEvent.newGuest = ssoWindow;
                ssoWindow.once('closed', () => __awaiter(this, void 0, void 0, function* () {
                    if (this.session) {
                        yield this.wipeSessionData();
                        yield SingleSignOn.protocol.unregister(this.session);
                    }
                    this.session = undefined;
                }));
                ssoWindow.on('page-title-updated', event => event.preventDefault());
                ssoWindow.webContents.on('new-window', event => event.preventDefault());
                ssoWindow.webContents.on('did-navigate', (event, url) => {
                    const { origin } = new url_1.URL(url);
                    if (origin.length > SingleSignOn.MAX_LENGTH_ORIGIN) {
                        event.preventDefault();
                    }
                    ssoWindow.setTitle(SingleSignOn.getWindowTitle(origin));
                });
                if (getLogger_1.ENABLE_LOGGING) {
                    ssoWindow.webContents.on('console-message', (_event, _level, message) => __awaiter(this, void 0, void 0, function* () {
                        const webViewId = lifecycle_1.getWebViewId(ssoWindow.webContents);
                        if (webViewId) {
                            const logFilePath = path.join(LOG_DIR, webViewId, config_1.config.logFileName);
                            try {
                                yield commons_1.LogFactory.writeMessage(message, logFilePath);
                            }
                            catch (error) {
                                console.error(`Cannot write to log file "${logFilePath}": ${error.message}`, error);
                            }
                        }
                    }));
                }
                return ssoWindow;
            };
            this.finalizeLogin = (type) => __awaiter(this, void 0, void 0, function* () {
                if (type === SingleSignOn.RESPONSE_TYPES.AUTH_SUCCESS) {
                    if (!this.session) {
                        yield this.dispatchResponse(SingleSignOn.RESPONSE_TYPES.AUTH_ERROR_SESS_NOT_AVAILABLE);
                        return;
                    }
                    try {
                        yield SingleSignOn.copyCookies(this.session, this.mainSession, this.windowOriginUrl);
                    }
                    catch (error) {
                        SingleSignOn.logger.warn(error);
                        yield this.dispatchResponse(SingleSignOn.RESPONSE_TYPES.AUTH_ERROR_COOKIE);
                        return;
                    }
                }
                yield this.dispatchResponse(type);
            });
            this.dispatchResponse = (type) => __awaiter(this, void 0, void 0, function* () {
                const isTypeValid = /^[A-Z_]{1,255}$/g;
                if (isTypeValid.test(type) === false) {
                    throw new Error('Invalid type detected, aborting.');
                }
                yield this.senderWebContents.executeJavaScript(`window.dispatchEvent(new MessageEvent('message', {origin: '${this.windowOriginUrl.origin}', data: {type: '${type}'}}));`);
                if (this.ssoWindow) {
                    this.ssoWindow.close();
                    this.ssoWindow = undefined;
                }
            });
            this.wipeSessionData = () => __awaiter(this, void 0, void 0, function* () {
                if (this.session) {
                    yield this.session.clearStorageData(undefined);
                }
            });
            this.mainBrowserWindow = mainBrowserWindow;
            this.windowOptions = windowOptions;
            this.senderEvent = senderEvent;
            this.senderWebContents = senderEvent.sender;
            this.mainSession = this.senderWebContents.session;
            this.windowOriginUrl = new url_1.URL(windowOriginURL);
        }
        static copyCookies(fromSession, toSession, url) {
            return __awaiter(this, void 0, void 0, function* () {
                const cookies = yield fromSession.cookies.get({ name: 'zuid' });
                for (const cookie of cookies) {
                    if (cookie.domain) {
                        yield toSession.cookies.set(Object.assign({ url: url.toString() }, cookie));
                    }
                }
                yield toSession.cookies.flushStore();
            });
        }
    }
    SingleSignOn.ALLOWED_BACKEND_ORIGINS = config_1.config.backendOrigins;
    SingleSignOn.PRELOAD_SSO_JS = path.join(electron_1.app.getAppPath(), config_1.config.electronDirectory, 'dist/renderer/preload-sso.js');
    SingleSignOn.SINGLE_SIGN_ON_FRAME_NAME = 'WIRE_SSO';
    SingleSignOn.SSO_PROTOCOL = 'wire-sso';
    SingleSignOn.SSO_PROTOCOL_HOST = 'response';
    SingleSignOn.SSO_PROTOCOL_RESPONSE_SIZE_LIMIT = 255;
    SingleSignOn.SSO_SESSION_NAME = 'sso';
    SingleSignOn.MAX_LENGTH_ORIGIN_DOMAIN = 255;
    SingleSignOn.MAX_LENGTH_ORIGIN = 'https://'.length + SingleSignOn.MAX_LENGTH_ORIGIN_DOMAIN;
    SingleSignOn.logger = getLogger_1.getLogger(path.basename(__filename));
    SingleSignOn.RESPONSE_TYPES = {
        AUTH_ERROR_COOKIE: 'AUTH_ERROR_COOKIE',
        AUTH_ERROR_SESS_NOT_AVAILABLE: 'AUTH_ERROR_SESS_NOT_AVAILABLE',
        AUTH_SUCCESS: 'AUTH_SUCCESS',
    };
    SingleSignOn.isSingleSignOnLoginWindow = (frameName) => SingleSignOn.SINGLE_SIGN_ON_FRAME_NAME === frameName;
    SingleSignOn.isBackendOrigin = (url) => {
        if (url === 'null') {
            return false;
        }
        return SingleSignOn.ALLOWED_BACKEND_ORIGINS.includes(new url_1.URL(url).origin);
    };
    SingleSignOn.getWindowTitle = (origin) => SingleSignOn.ALLOWED_BACKEND_ORIGINS.includes(origin) ? '' : origin;
    SingleSignOn.getWindowOpenerScript = () => {
        return `Object.defineProperty(window, 'opener', {
      configurable: true, // Needed on Chrome :(
      enumerable: false,
      value: Object.freeze({
        postMessage: ({type}) => {
          const params = new URLSearchParams();
          params.set('secret', '${SingleSignOn.loginAuthorizationSecret}');
          params.set('type', type);
          const url = new URL('${SingleSignOn.SSO_PROTOCOL}://${SingleSignOn.SSO_PROTOCOL_HOST}/');
          url.search = params.toString();
          document.location.href = url.toString();
        }
      }),
      writable: false,
    });`;
    };
    SingleSignOn.protocol = {
        generateSecret: (length) => {
            return new Promise((resolve, reject) => {
                crypto.randomBytes(length, (error, bytes) => {
                    if (error) {
                        return reject(error);
                    }
                    resolve(bytes.toString('hex'));
                });
            });
        },
        register: (session, finalizeLogin) => __awaiter(void 0, void 0, void 0, function* () {
            SingleSignOn.loginAuthorizationSecret = yield SingleSignOn.protocol.generateSecret(24);
            const handleRequest = (request, response) => {
                try {
                    const requestURL = new url_1.URL(request.url);
                    if (requestURL.protocol !== `${SingleSignOn.SSO_PROTOCOL}:`) {
                        throw new Error('Protocol is invalid');
                    }
                    if (requestURL.hostname !== SingleSignOn.SSO_PROTOCOL_HOST) {
                        throw new Error('Host is invalid');
                    }
                    if (typeof SingleSignOn.loginAuthorizationSecret !== 'string') {
                        throw new Error('Secret has not be set or has been consumed');
                    }
                    if (requestURL.searchParams.get('secret') !== SingleSignOn.loginAuthorizationSecret) {
                        throw new Error('Secret is invalid');
                    }
                    const type = requestURL.searchParams.get('type');
                    if (typeof type !== 'string') {
                        throw new Error('Response is empty');
                    }
                    if (type.length > SingleSignOn.SSO_PROTOCOL_RESPONSE_SIZE_LIMIT) {
                        throw new Error('Response type is too long');
                    }
                    finalizeLogin(type);
                    response('Please wait...');
                }
                catch (error) {
                    response(`An error happened, please close the window and try again. Error: ${error.toString()}`);
                }
            };
            const isHandled = yield session.protocol.isProtocolHandled(SingleSignOn.SSO_PROTOCOL);
            if (!isHandled) {
                session.protocol.registerStringProtocol(SingleSignOn.SSO_PROTOCOL, handleRequest, error => {
                    if (error) {
                        throw new Error(`Failed to register protocol. Error: ${error}`);
                    }
                });
            }
        }),
        unregister: (session) => {
            return new Promise((resolve, reject) => {
                session.protocol.unregisterProtocol(SingleSignOn.SSO_PROTOCOL, error => {
                    return error ? reject(error) : resolve();
                });
            });
        },
    };
    return SingleSignOn;
})();
exports.SingleSignOn = SingleSignOn;
//# sourceMappingURL=SingleSignOn.js.map